package hk.advanpro.cloud.api;

import org.apache.commons.lang3.StringUtils;

import java.lang.annotation.Annotation;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.HashMap;
import java.util.Map;

import hk.advanpro.cloud.api.annotation.FilePayload;
import hk.advanpro.cloud.api.annotation.FromPayload;
import hk.advanpro.cloud.api.annotation.Header;
import hk.advanpro.cloud.api.annotation.Headers;
import hk.advanpro.cloud.api.annotation.JsonPayload;
import hk.advanpro.cloud.api.annotation.PathVariable;
import hk.advanpro.cloud.api.annotation.RequestParam;
import hk.advanpro.cloud.api.annotation.Response;
import hk.advanpro.cloud.api.annotation.Responses;
import hk.advanpro.cloud.api.annotation.Url;
import hk.advanpro.cloud.api.callback.ResponseErrorCallback;
import hk.advanpro.cloud.api.callback.ResponseFailureCallback;
import hk.advanpro.cloud.api.client.CloudApi;
import hk.advanpro.cloud.api.interceptor.RequestInterceptor;
import hk.advanpro.cloud.api.interceptor.ResponseInterceptor;
import hk.advanpro.cloud.api.options.ParameterOptions;
import hk.advanpro.cloud.api.options.RequestOptions;
import okhttp3.HttpUrl;
import okhttp3.OkHttpClient;

/**
 * 云端接口动态代理对象
 * Created by yejianfei on 2017/8/12.
 */

public class CloudApiProxy implements InvocationHandler {

    private OkHttpClient client = new OkHttpClient();

    private Map<Method, RequestOptions> cache = new HashMap<>();

    private RequestOptions global;//公共请求参数
    private RequestOptions privateOption = new RequestOptions();//api私有的请求参数

    private String[] baseUrls;

    public CloudApiProxy(String baseUrl, RequestOptions global) {
        this.baseUrls = new String[]{baseUrl};
        this.global = global;

        if (this.global == null) {
            this.global = new RequestOptions();
        }
    }

    public CloudApiProxy(Class<? extends CloudApi> api, String[] baseUrls, RequestOptions global) {
        try {
            this.baseUrls = baseUrls;
            this.global = global;

            if (this.global == null) {
                this.global = new RequestOptions();
            }

            Responses responses = api.getAnnotation(Responses.class);
            for (int i = 0; responses != null && i < responses.value().length; i++) {
                Response response = responses.value()[i];
                global.getParameters().add(new ParameterOptions(
                        ParameterOptions.RESPONSE,
                        getResponseInterceptor(response.interceptor()),
                        getResponseFailureCallback(response.failure()),
                        getResponseErrorCallback(response.error()),
                        ParameterOptions.TYPE));
            }


            Headers headers = api.getAnnotation(Headers.class);
            for (int i = 0; headers != null && i < headers.value().length; i++) {
                Header payload = headers.value()[i];
                global.getParameters().add(new ParameterOptions(
                        0,
                        ParameterOptions.HEADER,
                        payload.name(),
                        payload.value(),
                        getRequestInterceptor(payload.interceptor()),
                        ParameterOptions.TYPE));
            }

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public Object invoke(Object o, Method method, Object[] objects) throws Throwable {
        RequestOptions options = null;
        Type type = method.getGenericReturnType();

        switch (method.getName()) {
            case "get":
            case "post":
            case "put":
            case "delete":
                options = new RequestOptions();
                options.setUrl(buildUrl(objects[0]));
                options.setMethod(method.getName().toUpperCase());

                String value = objects.length == 3 ? buildPayload(objects[1]) : null;

                options.getParameters().add(new ParameterOptions(
                        0,
                        ParameterOptions.STRING_PAYLOAD,
                        "",
                        value,
                        true,
                        ParameterOptions.METHOD));
                type = (Type) objects[objects.length == 3 ? 2 : 1];
                break;
            case "addHeader":
                options = privateOption;
                if (objects.length == 2 && objects[0] instanceof String) {
                    String name = (String) objects[0];
                    Object object = objects[1];
                    if (object instanceof String) {
                        options.getParameters().add(new ParameterOptions(0, ParameterOptions.HEADER, name, (String) object, ParameterOptions.TYPE));
                    } else if (object instanceof RequestInterceptor) {
                        options.getParameters().add(new ParameterOptions(0, ParameterOptions.HEADER, name, null, (RequestInterceptor) object, ParameterOptions.TYPE));
                    }
                }
                break;
            case "addResponseInterceptor":
                options = privateOption;
                if (objects.length == 1 && objects[0] instanceof ResponseInterceptor) {
                    options.getParameters().add(new ParameterOptions(
                            ParameterOptions.RESPONSE,
                            (ResponseInterceptor) objects[0],
                            null,
                            null,
                            ParameterOptions.TYPE));
                } else if (objects.length == 1 && objects[0] instanceof ResponseFailureCallback) {
                    options.getParameters().add(new ParameterOptions(
                            ParameterOptions.RESPONSE,
                            null,
                            (ResponseFailureCallback) objects[0],
                            null,
                            ParameterOptions.TYPE));
                } else if (objects.length == 1 && objects[0] instanceof ResponseErrorCallback) {
                    options.getParameters().add(new ParameterOptions(
                            ParameterOptions.RESPONSE,
                            null,
                            null,
                            (ResponseErrorCallback) objects[0],
                            ParameterOptions.TYPE));
                } else if (objects.length == 3) {
                    options.getParameters().add(new ParameterOptions(
                            ParameterOptions.RESPONSE,
                            (ResponseInterceptor) objects[0],
                            (ResponseFailureCallback) objects[1],
                            (ResponseErrorCallback) objects[2],
                            ParameterOptions.TYPE));
                }
                break;
            default:
                options = getRequestOptions(method);
        }
        //如果方法返回类型为void,直接返回null
        if (("void").equals(method.getReturnType().getName())) {
            return null;
        }
        Constructor c = method.getReturnType().getConstructor(
                String[].class, OkHttpClient.class, RequestOptions.class, Type.class, Object[].class);
        return c.newInstance(this.baseUrls, client, RequestOptions.merge(options, global, privateOption), type, objects);
    }

    private RequestOptions getRequestOptions(Method method) throws Exception {

        if (!cache.containsKey(method)) {
            Url url = method.getAnnotation(Url.class);
            if (url == null) {
                throw new UnsupportedOperationException("not found @Url");
            }

            RequestOptions options = new RequestOptions();
            options.setUrl(url.value());
            options.setMethod(url.method().name());

            Headers headers = method.getAnnotation(Headers.class);
            for (int i = 0; headers != null && i < headers.value().length; i++) {
                Header payload = headers.value()[i];
                options.getParameters().add(new ParameterOptions(
                        i,
                        ParameterOptions.HEADER,
                        payload.name(),
                        payload.value(),
                        getRequestInterceptor(payload.interceptor()),
                        ParameterOptions.METHOD));
            }

            Responses responses = method.getAnnotation(Responses.class);
            for (int i = 0; responses != null && i < responses.value().length; i++) {
                Response response = responses.value()[i];
                options.getParameters().add(new ParameterOptions(
                        ParameterOptions.RESPONSE,
                        getResponseInterceptor(response.interceptor()),
                        getResponseFailureCallback(response.failure()),
                        getResponseErrorCallback(response.error()),
                        ParameterOptions.METHOD));
            }

            Annotation[][] annotations = method.getParameterAnnotations();

            for (int i = 0; annotations != null && i < annotations.length; i++) {

                for (Annotation annotation : annotations[i]) {
                    if (annotation instanceof JsonPayload) {
                        JsonPayload payload = (JsonPayload) annotation;

                        options.getParameters().add(new ParameterOptions(
                                i,
                                ParameterOptions.JSON_PAYLOAD,
                                payload.value(),
                                StringUtils.isBlank(payload.value()),
                                ParameterOptions.PARAMETER));
                    }
                    if (annotation instanceof FromPayload) {
                        FromPayload payload = (FromPayload) annotation;
                        options.getParameters().add(new ParameterOptions(
                                i,
                                ParameterOptions.FROM_PAYLOAD,
                                payload.value(),
                                ParameterOptions.PARAMETER));
                    }
                    if (annotation instanceof FilePayload) {
                        FilePayload payload = (FilePayload) annotation;
                        options.getParameters().add(new ParameterOptions(
                                i,
                                ParameterOptions.FILE_PAYLOAD,
                                payload.value(),
                                ParameterOptions.PARAMETER));
                    }
                    if (annotation instanceof PathVariable) {
                        PathVariable payload = (PathVariable) annotation;
                        options.getParameters().add(new ParameterOptions(
                                i, ParameterOptions.PATH, payload.value(), ParameterOptions.PARAMETER));
                    }

                    if (annotation instanceof RequestParam) {
                        RequestParam payload = (RequestParam) annotation;
                        options.getParameters().add(new ParameterOptions(
                                i, ParameterOptions.PARAM, payload.value(), ParameterOptions.PARAMETER));
                    }

                    if (annotation instanceof Header) {
                        Header payload = (Header) annotation;
                        options.getParameters().add(new ParameterOptions(
                                i,
                                ParameterOptions.HEADER,
                                payload.name(),
                                payload.value(),
                                getRequestInterceptor(payload.interceptor()),
                                ParameterOptions.PARAMETER));
                    }
                }
            }

            cache.put(method, options);
        }

        return cache.get(method);
    }

    private ResponseErrorCallback getResponseErrorCallback(
            Class<? extends ResponseErrorCallback> clazz) throws Exception {
        if (clazz == null || clazz.isInterface()) {
            return null;
        }
        return clazz.newInstance();
    }

    private ResponseFailureCallback getResponseFailureCallback(
            Class<? extends ResponseFailureCallback> clazz) throws Exception {
        if (clazz == null || clazz.isInterface()) {
            return null;
        }
        return clazz.newInstance();
    }

    private ResponseInterceptor getResponseInterceptor(
            Class<? extends ResponseInterceptor> clazz) throws Exception {
        if (clazz == null || clazz.isInterface()) {
            return null;
        }
        return clazz.newInstance();
    }

    private RequestInterceptor getRequestInterceptor(Class<? extends RequestInterceptor> clazz) throws Exception {
        if (clazz == null || clazz.isInterface()) {
            return null;
        }
        return clazz.newInstance();
    }

    private String buildPayload(Object o) {
        return String.valueOf(o);
    }

    private String buildUrl(Object o) {
        if (o instanceof String) {
            return (String) o;
        }

        if (o instanceof HttpUrl) {
            return o.toString();
        }
        return null;
    }
}
