package hk.advanpro.cloud.api;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import hk.advanpro.android.core.log.ALogger;
import hk.advanpro.cloud.api.callback.ResponseFailureCallback;
import hk.advanpro.cloud.api.callback.ResponseSuccessCallback;
import hk.advanpro.cloud.api.interceptor.RequestInterceptor;
import hk.advanpro.cloud.api.options.ParameterOptions;
import hk.advanpro.cloud.api.options.RequestOptions;
import hk.advanpro.cloud.api.result.ResponseResult;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.HttpUrl;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

/**
 * 云端接口请求上下文环境对象，处理每次一次请求的参数及响应结果
 * Created by yejianfei on 2017/8/12.
 */

public class CloudRequestContext<T extends ResponseResult> {

    public static final int MAIN_THREAD = 0;

    public static final int NEW_THREAD = 1;

    private OkHttpClient client;

    private Request.Builder builder = new Request.Builder();

    private RequestOptions options;

    private Class<T> clazz;

    private Type[] types;

    String requestMessage;


    private ResponseSuccessCallback success;

    private ResponseFailureCallback failure;

    @SuppressWarnings("unchecked")
    public CloudRequestContext(String[] baseUrls, OkHttpClient client, RequestOptions options, Type returnType, Object[] args) throws Exception {
        this.client = client;
        this.options = options;

        if (returnType instanceof ParameterizedType && !(((ParameterizedType) returnType).getActualTypeArguments()[0] instanceof Class)) {
            this.types = ((ParameterizedType) ((ParameterizedType) returnType).getActualTypeArguments()[0]).getActualTypeArguments();
            this.clazz = (Class) ((ParameterizedType) ((ParameterizedType) returnType).getActualTypeArguments()[0]).getRawType();
        } else if (returnType instanceof ParameterizedType) {
            this.clazz = (Class) ((ParameterizedType) returnType).getActualTypeArguments()[0];
        } else if (returnType instanceof Class) {
            this.clazz = (Class) returnType;
        }
        initBuilder(baseUrls, this.builder, options, args);
    }


    public CloudRequestContext<T> addHeader(String name, String value) {
        this.builder.addHeader(name, value);
        return this;
    }

    public CloudRequestContext<T> removeHeader(String name) {
        this.builder.removeHeader(name);
        return this;
    }

    public CloudRequestContext<T> header(String name, String value) {
        this.builder.header(name, value);
        return this;
    }

    @SuppressWarnings("unchecked")
    public T await() throws Exception {
        Call call = client.newCall(this.builder.build());
        try {
            Response response = call.execute();
            interceptResponse(this.options, call, response);

            T t = this.clazz.newInstance();
            t.parse(this.types, response);

            if (success != null && t.isSuccess()) {
                // 请求成功
                success.call(t);
            }

            if (failure != null && !t.isSuccess()) {
                // 不成功
                ALogger.e("cloud_request", requestMessage + String.format(" ,result code=%d,message=%s", t.getCode(), t.getMsg()));
                failure.call(null, t);
            }

            return t;
        } catch (IllegalAccessException | InstantiationException e) {
            interceptResponseError(CloudRequestContext.this.options, call, e);
            if (failure != null) {
                failure.call(e, null);
            } else {
                throw e;
            }

            return null;
        }
    }

    public void exec() {
        client.newCall(this.builder.build()).enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                interceptResponseError(CloudRequestContext.this.options, call, e);
                if (failure != null) {
                    failure.call(e, null);
                }
            }

            @SuppressWarnings("unchecked")
            @Override
            public void onResponse(Call call, Response response) {
                try {
                    interceptResponse(CloudRequestContext.this.options, call, response);

                    T t = CloudRequestContext.this.clazz.newInstance();
                    t.parse(CloudRequestContext.this.types, response);

                    if (success != null && t.isSuccess()) {
                        success.call(t);
                    }

                    if (failure != null && !t.isSuccess()) {
                        ALogger.e("cloud_request", requestMessage + String.format(" ,request code=%d,message=%s", t.getCode(), t.getMsg()));
                        failure.call(null, t);
                    }

                } catch (Exception e) {
                    interceptResponseError(CloudRequestContext.this.options, call, e);
                    if (failure != null) {
                        failure.call(e, null);
                    }
                }
            }
        });
    }

    public CloudRequestContext<T> onSuccess(ResponseSuccessCallback cb) {
        this.success = cb;
        return this;
    }

    public CloudRequestContext<T> onFailure(ResponseFailureCallback cb) {
        this.failure = cb;
        return this;
    }

    protected void initBuilder(String[] baseUrls, Request.Builder builder, RequestOptions options, Object[] args) throws Exception {
        MediaType media = MediaType.parse("application/json; charset=utf-8");
        JSONObject json = new JSONObject();
        String content = null;
        MultipartBody.Builder multipartBodyBuilder = null;
        HashMap<String, String> params = new HashMap<>();
        String url = options.getUrl();

        for (ParameterOptions o : options.getParameters()) {
            Object obj = args != null && args.length > 0 && o.getIndex() < args.length ? args[o.getIndex()] : null;

            switch (o.getType()) {
                case ParameterOptions.HEADER:
                    // 使用参数获取请求头。
                    if (o.getTarget() == ParameterOptions.PARAMETER && obj != null) {
                        builder.addHeader(o.getName(), valueOf(obj));
                    }

                    // 使用常量值获取请求头。
                    if (StringUtils.isNotBlank(o.getValue()) && o.getTarget() != ParameterOptions.PARAMETER) {
                        builder.addHeader(o.getName(), o.getValue());
                    }

                    // 使用拦截器实例获取请求头
                    if (o.getRequestInterceptor() != null && o.getTarget() != ParameterOptions.PARAMETER) {
                        o.getRequestInterceptor().invoke(RequestInterceptor.HEADER, builder, o, args);
                    }

                    break;
                case ParameterOptions.JSON_PAYLOAD:
                    if (o.isRoot() && obj != null) {
                        content = JSON.toJSONString(obj);
                    } else if (obj != null) {
                        json.put(o.getName(), obj);
                    }

                    break;
                case ParameterOptions.FROM_PAYLOAD:
                    if (multipartBodyBuilder == null) {
                        multipartBodyBuilder = new MultipartBody.Builder();
                        multipartBodyBuilder.setType(MultipartBody.FORM);
                    }
                    multipartBodyBuilder.addFormDataPart(o.getName(), valueOf(obj));
                    break;
                case ParameterOptions.FILE_PAYLOAD:
                    if (multipartBodyBuilder == null) {
                        multipartBodyBuilder = new MultipartBody.Builder();
                        multipartBodyBuilder.setType(MultipartBody.FORM);
                    }
                    File file = (File) obj;
                    multipartBodyBuilder.addFormDataPart(o.getName(), file.getName(), RequestBody.create(null, file));
                    break;
                case ParameterOptions.STRING_PAYLOAD:
                    if (o.getTarget() == ParameterOptions.METHOD || o.getTarget() == ParameterOptions.TYPE) {
                        content = o.getValue();
                    } else {
                        content = valueOf(obj);
                    }
                case ParameterOptions.PATH:
                    Matcher pathMatcher = Pattern.compile("(?<=\\{)(.+?)(?=\\})").matcher(url);
                    while (pathMatcher.find()) {
                        if (o.getName().equals(pathMatcher.group()) && obj != null) {
                            url = url.replace("{" + pathMatcher.group() + "}", valueOf(obj));
                        }
                    }
                    break;
                case ParameterOptions.PARAM:
                    params.put(o.getName(), valueOf(obj));
                    break;
                default:
            }
        }

        String baseUrl = baseUrls[0];
        if (baseUrl.endsWith("/")) {
            baseUrl = baseUrl.substring(0, baseUrl.length() - 1);
        }

        if (!url.startsWith("/")) {
            url = "/" + url;
        }

        HttpUrl httpUrl = HttpUrl.parse(baseUrl + url);
        if (httpUrl == null) {
            throw new NullPointerException();
        }

        HttpUrl.Builder urlBuilder = httpUrl.newBuilder();
        for (String key : params.keySet()) {
            urlBuilder.addEncodedQueryParameter(key, URLEncoder.encode(params.get(key), "utf-8"));
        }

        builder.url(urlBuilder.toString());

        requestMessage = String.format("http request method = %s, url = %s ,body = %s", options.getMethod(), urlBuilder.toString(),
                StringUtils.isNotBlank(content) ? content : (json != null ? json.toJSONString() : ""));
        System.out.println("--> " + requestMessage);
        RequestBody body;
        if (multipartBodyBuilder != null) {
            body = multipartBodyBuilder.build();
        } else {
            body = RequestBody.create(media, StringUtils.isNotBlank(content) ? content : json.toJSONString());
        }
        switch (options.getMethod()) {
            case "POST":
                builder.post(body);
                break;
            case "PUT":
                builder.put(body);
                break;
            case "DELETE":
                builder.delete(body);
                break;
            default:
        }
    }

    private void interceptResponse(RequestOptions options, Call call, Response response) {
        for (ParameterOptions o : options.getParameters()) {
            switch (o.getType()) {
                case ParameterOptions.RESPONSE:
                    if (o.getResponseInterceptor() != null) {
                        o.getResponseInterceptor().invoke(call, response, null);
                    }

                    // http错误
                    if (o.getErrorCallback() != null && !response.isSuccessful()) {
                        o.getErrorCallback().call(response, null);
                    }
                    break;
                default:
            }
        }
    }

    private void interceptResponseError(RequestOptions options, Call call, Throwable throwable) {
        for (ParameterOptions o : options.getParameters()) {
            switch (o.getType()) {
                case ParameterOptions.RESPONSE:
                    if (o.getResponseInterceptor() != null) {
                        o.getResponseInterceptor().invoke(call, null, throwable);
                    }

                    // 异常
                    if (o.getErrorCallback() != null) {
                        o.getErrorCallback().call(null, throwable);
                    }
                    break;
                default:
            }
        }
    }

    private String valueOf(Object o) {
        return String.valueOf(o);
    }

}
