package hk.advanpro.cloud.api.client;

import java.util.ArrayList;

import hk.advanpro.cloud.api.CloudRequestContext;
import hk.advanpro.cloud.api.annotation.Header;
import hk.advanpro.cloud.api.annotation.Headers;
import hk.advanpro.cloud.api.annotation.JsonPayload;
import hk.advanpro.cloud.api.annotation.PathVariable;
import hk.advanpro.cloud.api.annotation.RequestMethod;
import hk.advanpro.cloud.api.annotation.RequestParam;
import hk.advanpro.cloud.api.annotation.Url;
import hk.advanpro.cloud.api.payload.AccountAuthPayload;
import hk.advanpro.cloud.api.payload.QuickLoginPayload;
import hk.advanpro.cloud.api.result.RestfulResult;
import hk.advanpro.cloud.api.result.uc.AccountAuthResult;
import hk.advanpro.cloud.api.result.uc.AccountExistResult;
import hk.advanpro.cloud.api.result.uc.CheckVCodeResult;
import hk.advanpro.cloud.api.result.uc.LoginResult;

/**
 * 用于云端用户中心的访问接口定义。
 * Created by yejianfei on 2017/8/12.
 */
@Headers({
        @Header(name = "Content-Type", value = "application/json")
})
public interface DevUCApi extends CloudApi {
    public static final String NAME = "dev";

    /**
     * 快速登录
     *
     * @param payload
     * @return 请求上下文
     */
    @Url(value = "/developers/quicklogin", method = RequestMethod.POST)
    CloudRequestContext<RestfulResult<LoginResult>> quickLoginByPhone(@JsonPayload() QuickLoginPayload payload);

    /**
     * 用户登录请求接口。
     *
     * @param account 登录账号
     * @param passwd  登录密码
     * @param lang    语言类型
     * @return 请求上下文环境
     */
    @Url(value = "/developers/login", method = RequestMethod.POST)
    CloudRequestContext<RestfulResult<LoginResult>> login(
            @JsonPayload("account") String account,
            @JsonPayload("passwd") String passwd,
            @JsonPayload("lang") String lang);

    /**
     * 发送验证码请求接口
     *
     * @param account 手机或邮箱账号（使用限制模式时将忽略该参数）
     * @param type    验证码类型(0:账号是手机 1:账号是邮箱)
     * @param limit   限制模式（忽略输入的手机或邮箱账号，使用用户注册的手机或邮箱发送）
     * @param lang    语言类型
     * @return 请求上下文环境
     */
    @Url(value = "/developers/vcode", method = RequestMethod.POST)
    CloudRequestContext<RestfulResult> sendVCode(
            @JsonPayload("account") String account,
            @JsonPayload("type") int type,
            @JsonPayload("limit") boolean limit,
            @JsonPayload("lang") String lang);

    /**
     * 判断验证码是否有效请求接口。
     * （忽略输入的手机或邮箱账号，使用用户注册的手机或邮箱发送）
     *
     * @param code    待验证的验证码
     * @param account 非限制模式使用的手机及邮箱地址
     * @return 请求上下文环境
     */
    @Url(value = "/developers/vcode/{code}", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<CheckVCodeResult>> checkVCode(
            @PathVariable("code") String code,
            @RequestParam("account") String account);

    /**
     * 忘记密码请求接口。
     *
     * @param account 手机、邮箱地址及微信unionid等
     * @param passwd  新密码
     * @param vCode   验证码
     * @return 请求上下文环境
     */
    @Url(value = "/developers/forgot", method = RequestMethod.POST)
    CloudRequestContext<RestfulResult> forgot(
            @JsonPayload("account") String account,
            @JsonPayload("passwd") String passwd,
            @JsonPayload("v_code") String vCode);

    /**
     * 判断用户是否存在请求接口。
     *
     * @param account 手机、邮箱地址及微信unionid等
     * @return 请求上下文环境
     */
    @Url(value = "/developers/users/{account}/exist", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<AccountExistResult>> accountExist(
            @PathVariable("account") String account);

    /**
     * 判断用户是否设置密码
     *
     * @return 请求上下文环境
     */
    @Url(value = "/developers/my/passwd", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<Boolean>> isSetPassword();


    /**
     * 修改登录密码的请求接口
     *
     * @param passwd    当前密码
     * @param newPasswd 新密码
     * @return 请求上下文
     */
    @Url(value = "/developers/my/passwd", method = RequestMethod.PUT)
    CloudRequestContext<RestfulResult> changeMyPasswd(
            @JsonPayload("passwd") String passwd,
            @JsonPayload("new_passwd") String newPasswd);


    /**
     * 绑定或换邦登录账号的请求接口
     *
     * @param payload 绑定或换邦登录账号基本信息
     * @return 请求上下文
     */
    @Url(value = "/developers/my/account/auths", method = RequestMethod.POST)
    CloudRequestContext<RestfulResult> accountAuths(@JsonPayload() AccountAuthPayload payload);

    /**
     * 解绑登录账号的请求接口
     *
     * @param name 账号名称
     * @return 请求上下文
     */
    @Url(value = "/developers/my/account/auths/{name}", method = RequestMethod.DELETE)
    CloudRequestContext<RestfulResult> delAccountAuths(@PathVariable("name") String name);

    /**
     * 获取绑定账号列表的请求接口
     *
     * @return 请求上下文
     */
    @Url(value = "/developers/my/account/auths", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<ArrayList<AccountAuthResult>>> accountAuthsList();


}
