package hk.advanpro.cloud.api.client;

import java.io.File;
import java.util.List;

import hk.advanpro.cloud.api.CloudRequestContext;
import hk.advanpro.cloud.api.annotation.FilePayload;
import hk.advanpro.cloud.api.annotation.FromPayload;
import hk.advanpro.cloud.api.annotation.Header;
import hk.advanpro.cloud.api.annotation.Headers;
import hk.advanpro.cloud.api.annotation.PathVariable;
import hk.advanpro.cloud.api.annotation.RequestMethod;
import hk.advanpro.cloud.api.annotation.Url;
import hk.advanpro.cloud.api.result.RestfulResult;
import hk.advanpro.cloud.api.result.oss.CountrysResult;
import hk.advanpro.cloud.api.result.oss.FileUploadResult;
import hk.advanpro.cloud.api.result.oss.GetAreasResult;
import hk.advanpro.cloud.api.result.oss.GetAreasV2Result;
import hk.advanpro.cloud.api.result.oss.OssPathInfoResult;
import hk.advanpro.cloud.api.result.oss.ProdectVersionResult;
import hk.advanpro.cloud.api.result.oss.STSResult;
import hk.advanpro.cloud.api.result.oss.TownsResult;

/**
 * 用于云端 OSS 的访问接口定义。
 * Created by jalin on 2017/9/20.
 */

@Headers({
        @Header(name = "Content-Type", value = "application/json")
})
public interface OSSApi extends CloudApi {
    public static final String NAME = "oss";

    /**
     * 获取文件目录的OSS信息
     *
     * @param path 数据类别,上传临时文件时所使用的文件目录
     * @return 请求上下文环境
     */
    @Url(value = "/{path}/info", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<OssPathInfoResult>> getFilePathOssInfo(@PathVariable("path") String path);


    /**
     * 获取文件目录STS授权
     *
     * @param path 上传临时文件时所使用的文件目录
     * @return 请求上下文环境
     */
    @Url(value = "/{path}/token", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<STSResult>> getFilePathStsAccess(@PathVariable("path") String path);

    /**
     * 上传图片
     *
     * @param mode 存储方式。temp ＝ 临时存储，1小时后文件自动删除，需要调用提交接口进行永久存储。durable ＝ 永久存储，直接存储至阿里云OSS。
     * @param file 上传文件路径
     * @return 请求上下文环境
     */
    @Url(value = "/photos", method = RequestMethod.POST)
    CloudRequestContext<RestfulResult<FileUploadResult>> uploadPhoto(@FromPayload("mode") String mode, @FilePayload("file") File file);


    /**
     * 上传图片
     *
     * @param path 存储的位置.
     * @param mode 存储方式。temp ＝ 临时存储，1小时后文件自动删除，需要调用提交接口进行永久存储。durable ＝ 永久存储，直接存储至阿里云OSS。
     * @param file 上传文件路径
     * @return 请求上下文环境
     */
    @Url(value = "/{path}", method = RequestMethod.POST)
    CloudRequestContext<RestfulResult<FileUploadResult>> uploadPhoto(@PathVariable("path") String path, @FromPayload("mode") String mode, @FilePayload("file") File file);

    /**
     * 上传头像
     *
     * @param mode 模式 temp=临时, durable=永久
     * @param file 上传文件路径
     * @return 请求上下文环境
     */
    @Url(value = "/profiles", method = RequestMethod.POST)
    CloudRequestContext<RestfulResult<FileUploadResult>> uploadProfile(@FromPayload("mode") String mode, @FilePayload("file") File file);


    /**
     * 获取最新固件版本信息
     *
     * @param product 设备类型 如:好好呼吸：LF1.1（去前两位）
     * @return
     */
    @Url(value = "/firmwares/{product}/latest.json", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<ProdectVersionResult>> getFirmwareVersionInfo(@PathVariable("product") String product);

    /**
     * 获取最新APP版本信息
     *
     * @param product APP类型 ，如好好呼吸APP:laf
     * @return
     */
    @Url(value = "/apps/{product}/android/latest.json", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<ProdectVersionResult>> getAppVersionInfo(@PathVariable("product") String product);


    /**
     * 获取省市区
     *
     * @return
     */
    @Url(value = "/areas/list.json", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<List<GetAreasResult>>> getAreas();


    /**
     * 获取世界各个国家
     *
     * @param language:string:语言 - zh_CN:简体中文
     *                           - zh_TW:繁体中文
     *                           - en:英文
     * @return
     */
    @Url(value = "/areas/{language}/country.json", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<List<CountrysResult>>> getCountrys(@PathVariable("language") String language);

    /**
     * 根据国家编码获取省市区
     *
     * @param language
     * @param country_code
     * @return
     */
    @Deprecated
    @Url(value = "/areas/{language}/country/{country_code}/list.json", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<List<GetAreasResult>>> getAreas(@PathVariable("language") String language,
                                                                      @PathVariable("country_code") String country_code);

    /**
     * 根据国家编码获取省市区
     *
     * @param language
     * @param country_code
     * @return
     */
    @Url(value = "/areas/{language}/country/{country_code}/list.json", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<List<GetAreasV2Result>>> getAreasV2(@PathVariable("language") String language,
                                                                          @PathVariable("country_code") String country_code);

    /**
     * 根据区编码获取街道
     *
     * @param language
     * @param country_code
     * @param code
     * @return
     */
    @Url(value = "/areas/{language}/country/{country_code}/towns/{code}.json", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<List<TownsResult>>> getTowns(@PathVariable("language") String language,
                                                                   @PathVariable("country_code") String country_code,
                                                                   @PathVariable("code") String code);


    /**
     * 拉玛宠物推荐留言
     *
     * @return
     */
    @Url(value = "/wikipedia/text/lamaze/recommend_pet_message.json", method = RequestMethod.GET)
    CloudRequestContext<RestfulResult<List<String>>> getRecommendPetMessage();

}
